/**
 *        __       __               __ 
 *   ____/ /_ ____/ /______ _ ___  / /_
 *  / __  / / ___/ __/ ___/ / __ `/ __/
 * / /_/ / (__  ) / / /  / / /_/ / / 
 * \__,_/_/____/_/ /_/  /_/\__, /_/ 
 *                           / / 
 *                           \/ 
 * http://distriqt.com
 *
 * @file   		NotificationsContext.java
 * @brief  		Main Context for this extension
 * @author 		Michael Archbold (ma@distriqt.com)
 * @created		Jan 16, 2012
 * @updated		$Date:$
 * @copyright	http://distriqt.com/copyright/license.txt
 */
package com.distriqt.extension.notifications;

import java.util.HashMap;
import java.util.Map;

import android.content.IntentFilter;

import com.adobe.fre.FREContext;
import com.adobe.fre.FREFunction;
import com.distriqt.extension.notifications.functions.NotificationsCancelAllFunction;
import com.distriqt.extension.notifications.functions.NotificationsImplementationFunction;
import com.distriqt.extension.notifications.functions.NotificationsIsSupportedFunction;
import com.distriqt.extension.notifications.functions.NotificationsSetBadgeNumberFunction;
import com.distriqt.extension.notifications.functions.NotificationsVersionFunction;
import com.distriqt.extension.notifications.functions.NotificationsCancelFunction;
import com.distriqt.extension.notifications.functions.NotificationsNotifyFunction;

public class NotificationsContext extends FREContext 
{
	public static String VERSION		= "1.0";
	public static String IMPLEMENTATION	= "Android";
	
	public NotificationsReceiver notificationsReceiver = null;
	
	@Override
	public void dispose() 
	{
		if (notificationsReceiver != null)
		{
			// TODO:: Release this?
		}
	}

	@Override
	public Map<String, FREFunction> getFunctions() 
	{
		Map<String, FREFunction> functionMap = new HashMap<String, FREFunction>();
		
		functionMap.put( "isSupported",		new NotificationsIsSupportedFunction() );
		functionMap.put( "version", 		new NotificationsVersionFunction() );
		functionMap.put( "implementation", 	new NotificationsImplementationFunction() );
		functionMap.put( "notify",  		new NotificationsNotifyFunction() );
		functionMap.put( "cancel", 			new NotificationsCancelFunction() );
		functionMap.put( "cancelAll", 		new NotificationsCancelAllFunction() );
		functionMap.put( "setBadgeNumber",  new NotificationsSetBadgeNumberFunction() );
		
		return functionMap;
	}

	
	public void registerReceiver()
	{
		if (notificationsReceiver == null)
		{
			notificationsReceiver = new NotificationsReceiver( this );
			
			IntentFilter intentFilter = new IntentFilter( NotificationsReceiver.NOTIFICATION_ACTION );
			intentFilter.addDataScheme("id");
			getActivity().getApplicationContext().registerReceiver( notificationsReceiver, intentFilter );
		}
	}
	
}
