/**
 *        __       __               __ 
 *   ____/ /_ ____/ /______ _ ___  / /_
 *  / __  / / ___/ __/ ___/ / __ `/ __/
 * / /_/ / (__  ) / / /  / / /_/ / / 
 * \__,_/_/____/_/ /_/  /_/\__, /_/ 
 *                           / / 
 *                           \/ 
 * http://distriqt.com
 *
 * @file   		NotificationsNotifyFunction.java
 * @brief  		Notify Function for this extension
 * @author 		Michael Archbold (ma@distriqt.com)
 * @created		Jan 16, 2012
 * @updated		$Date:$
 * @copyright	http://distriqt.com/copyright/license.txt
 */
package com.distriqt.extension.notifications.functions;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.util.Log;

import com.adobe.fre.FREASErrorException;
import com.adobe.fre.FREContext;
import com.adobe.fre.FREFunction;
import com.adobe.fre.FREInvalidObjectException;
import com.adobe.fre.FRENoSuchNameException;
import com.adobe.fre.FREObject;
import com.adobe.fre.FRETypeMismatchException;
import com.adobe.fre.FREWrongThreadException;

import com.distriqt.extension.notifications.NotificationsContext;
import com.distriqt.extension.notifications.NotificationsExtension;
import com.distriqt.extension.notifications.NotificationsReceiver;
import com.distriqt.extension.notifications.events.NotificationsEvents;

public class NotificationsNotifyFunction implements FREFunction 
{
	private static String TAG = NotificationsNotifyFunction.class.getSimpleName();
	
	NotificationsReceiver notificationsReceiver = null;
	
	@Override
	public FREObject call(FREContext context, FREObject[] passedArgs) 
	{
		NotificationsContext nc = (NotificationsContext)context;
		nc.registerReceiver();

		Activity activity  = nc.getActivity();
		Context appContext = activity.getApplicationContext();
		
		int id = 0;
		String tickerText 	= "";
		String title 		= "";
		String body 		= "";
		String iconType		= "";
		int count			= 1;
		Boolean vibrate		= false;
		Boolean playSound	= false;
		int flags			= 0;
		int delay 			= 0;
		String data			= "";
		
		Log.i( TAG, "notify" );
		
		try
		{
			//
			//	Process the arguments
			FREObject fro = passedArgs[0];
	        id = fro.getAsInt();

	        FREObject asNotification = passedArgs[1];
	        
	        tickerText 	= asNotification.getProperty( "tickerText" ).getAsString();
	        title 		= asNotification.getProperty( "title" ).getAsString();
	        body 		= asNotification.getProperty( "body" ).getAsString();
	        iconType	= asNotification.getProperty( "iconType" ).getAsString();
	        count		= asNotification.getProperty( "count" ).getAsInt();
	        vibrate		= asNotification.getProperty( "vibrate" ).getAsBool();
	        playSound	= asNotification.getProperty( "playSound" ).getAsBool();
	        flags		= asNotification.getProperty( "flags" ).getAsInt();
	        delay		= asNotification.getProperty( "delay" ).getAsInt();
	        data 		= asNotification.getProperty( "data" ).getAsString();
	        
			long when = System.currentTimeMillis() + delay*1000;
			
			//
			//	Determine the icon to use
			int icon;
			
			if (iconType.equals("calendar")) 			icon = nc.getResourceId( "drawable.notification_calendar" );
			else if (iconType.equals("document"))		icon = nc.getResourceId( "drawable.notification_document" );
			else if (iconType.equals("error"))			icon = nc.getResourceId( "drawable.notification_error" );
			else if (iconType.equals("flag"))			icon = nc.getResourceId( "drawable.notification_flag" );
			else if (iconType.equals("mail") )			icon = nc.getResourceId( "drawable.notification_mail" );
			else if (iconType.equals("preferences")) 	icon = nc.getResourceId( "drawable.notification_preferences" );
			else if (iconType.equals("star")) 			icon = nc.getResourceId( "drawable.notification_star" );
			else 										icon = nc.getResourceId( "drawable.icon" );


			//
			//	Setup the response Intent
			Intent notificationIntent = new Intent( NotificationsReceiver.NOTIFICATION_ACTION );
			notificationIntent.putExtra( "when", when );
			notificationIntent.putExtra( "data", data );
			notificationIntent.putExtra( "id", id );
			notificationIntent.setData( Uri.parse( "id://"+id ) );
			notificationIntent.setFlags( Intent.FLAG_ACTIVITY_REORDER_TO_FRONT | Intent.FLAG_ACTIVITY_SINGLE_TOP );
			PendingIntent contentIntent = PendingIntent.getBroadcast( appContext, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT );
			
			//
			// Create the notification
			Notification notification = new Notification( icon, tickerText, when );
			notification.setLatestEventInfo( appContext, title, body, contentIntent );
			notification.number = count;
			if (vibrate) 	notification.defaults |= Notification.DEFAULT_VIBRATE;
			if (playSound) 	notification.defaults |= Notification.DEFAULT_SOUND;
			notification.flags |= flags;
			
			
			//
			//	Look up the notification manager service and send the notification
			NotificationManager notificationManager = (NotificationManager) activity.getSystemService( Context.NOTIFICATION_SERVICE );

			notificationManager.notify( id, notification );
			
			
			//
			//
			NotificationsExtension.context.dispatchStatusEventAsync( NotificationsEvents.NOTIFICATION_DISPLAYED, String.format("%08X", id ) + data );
			
		}
		catch (IllegalStateException e)
		{
			e.printStackTrace();
		} 
		catch (FRETypeMismatchException e) 
		{
			e.printStackTrace();
		} 
		catch (FREInvalidObjectException e) 
		{
			e.printStackTrace();
		} 
		catch (FREWrongThreadException e)
		{
			e.printStackTrace();
		} 
		catch (FREASErrorException e) 
		{
			e.printStackTrace();
		} 
		catch (FRENoSuchNameException e) 
		{
			e.printStackTrace();
		}
		
		return null;
	}

}
